package plugin;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;

import plop.astro.Astrocoo;
import plop.astro.Astroframe;
import plop.astro.Astrotime;
import plop.astro.FK5;
import plop.astro.Unit;

//import cds.aladin.*;
import cds.aladin.Aladin;
import cds.aladin.AladinData;
import cds.aladin.AladinException;
import cds.aladin.AladinPlugin;
import cds.aladin.Obj;
import cds.tools.parser.Deg2Rad;

public final class EpochPlug extends AladinPlugin {


	private EpochWindow epochWin;
	private boolean DEBUG = false;

	public String menu()        { return "Epoch"; }
	public String description() {
		return "PLUGIN TUTORIAL:\n" +
		"Ce plugin permet de recalculer les positions des objets à une certaine époque," +
		" pour obtenir l'époque d'une plaque photo il faut la mettre dans le stack et la selection à l'aide du menu deroulant.";
	}   
	//   public String category()    { return "Plugin tutorial/Catalog"; }
	public String version()     { return "1.0 - june 2011"; }
	public String author()      { return "Guillaume Mahler"; }
	//   public String url()         { return "http://aladin.u-strasbg.fr/java/Plugins/CatalogManipulationPlug.java"; }


	public AladinData getAladinData() throws AladinException {
		return aladin.getAladinData();
	}

	// La methode exec qui va etre la methode executer par le programme
	public void exec() throws AladinException {
		this.epochWin = new EpochWindow(this);
	}

	// Doit retourner l'�poque courante => � modifier par l'utilisateur via l'interface
	private double getEpoch() { return RecupYear.recupYear(); }


	//Cette methode permet d'obtenir le nom de tous les plans catalogue du stack sous forme d'un tableau de String	
	/**
	 * 
	 * @return String[] string table of names of all catalogs in Aladin stack
	 */
	public String[] getCatName() throws AladinException {
		String[] planeNames = aladin.getAladinStack();
		final List<String> l = new ArrayList<String>();
		for (final String s : planeNames) {
			AladinData plane = aladin.getAladinData(s);
			if (plane.getPlaneType().equals("Overlay/Catalog") && plane.isReady()) {
				l.add(s);
			}}
		return l.toArray(new String[0]);}

	//Cette methode permet de choisir le plan catalogue souhaité en rentrant son nom en argument, elle retourne un objet aladin qui est le catalogue concerné
	/**
	 * Find a catalog plane with it name
	 * @param np the name of the catalog plane which want to be find 
	 * @return ad Aladin object containing the catalog plane 
	 */
	public AladinData getCat(String np) throws AladinException{
		AladinData ad = aladin.getAladinData(np);
		return (ad);}

	//Cette methode permet de recuper le nom du catalogue selectionné par l'utilisateur dans le menu deroulant
	/**
	 * @return the name of the Catalog which selected in the combox
	 */
	public String getNome() {
		return (String) epochWin.combo.getSelectedItem();
	}

	//Cette methode permet de recuper le nom de tous lesplans Image contenue danbs le stack Aladin et retourne un tableau de String
	/**
	 * 
	 * @return String[] string table of names of all Image plane in Aladin stack
	 */
	public String[] getImName() throws AladinException {
		String[] planeNames = aladin.getAladinStack();
		final List<String> l = new ArrayList<String>();
		for (final String s : planeNames) {
			AladinData plane = aladin.getAladinData(s);
			if (plane.getPlaneType().equals("Image") && plane.isReady()) {
				l.add(s);}}
		return l.toArray(new String[0]);}

	//Cette methode permet de recuper le double ( qui est censé etre la date ) derrière le mots Fits EPOCH du header Fits de l'image
	/**
	 * this methods return the date of the image plate which is enter in parameter
	 * @param ni String containing the name of image plane 
	 * @return epoch a double containing the image plate epoch 
	 */
	public double getImageEpoch(String ni) throws AladinException{
		AladinData ad = aladin.getAladinData(ni);
		double epoch = Double.parseDouble(ad.seeFitsKeys().get("EPOCH").toString());
		//Ce bout de code permet de recuper tous les FitsKeys du header du plan image rentré en parametre
		/*Iterator<Map.Entry<?, ?>>  it = ad.seeFitsKeys().entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry<?, ?> e = it.next();
			System.out.println("Key = " + e.getKey() + " ; value = " + e.getValue());
		}*/
		return epoch;
	}

	//Cette methode prend en parametre un double(année Julian) et renvoie une String comprenant le jour julian
	/**
	 * @param double jj a Julian year
	 * @return jd String, the conversion of Julian year into Julian Day
	 */
	public String JtoJD(double jj) {
		String jd =Double.toString(Astrotime.J2JD(jj));
		return jd;
	}

	//Cette methode prend en parametre un double(année Julian) et renvoie une String comprenant le jour Julian modifié JD = 2400000.5+MJD
	/**
	 * @param double jj a Julian year
	 * @return mjd String, the conversion of Julian year into Modified Julian Day
	 */
	public String JtoMJD(double jj) {
		String mjd =Double.toString(Astrotime.JD2MJD(Astrotime.J2JD(jj)));
		return mjd;
	}

	//Cette methode prend une année julian en parametre et retourne un objet de type String contenant la date iso8601 
	/**
	 * @param double jj a Julian year
	 * @return jiso String, the conversion of Julian year into Calendar date formated in iso8601
	 */
	public String J2ymdhms(double y) {
		//Cette methode prend normalement des jours Julian je reconvertis donc l'année julian rentrer en parametre en jour Julian
		y=Astrotime.J2JD(y);
		int[] ymd = new int[4];
		NumberFormat nf=NumberFormat.getInstance(); // Get Instance of NumberFormat
		nf.setMinimumIntegerDigits(2);  // The minimum Digits required is 2
		nf.setMaximumIntegerDigits(2); // The maximum Digits required is 2


		Astrotime.JD2YMD(y, ymd);
		Calendar date = Calendar.getInstance();
		date.setTimeZone(TimeZone.getTimeZone("GMT+00"));
		date.setTimeInMillis(Astrotime.JD2ms(y));

		String jiso = String.valueOf(ymd[0])+ "-" +  nf.format((ymd[1]+1))+ "-" + nf.format(ymd[2]) + "T" + nf.format(date.get(Calendar.HOUR_OF_DAY)) + ":" + nf.format(date.get(Calendar.MINUTE)) + ":" + nf.format(date.get(Calendar.SECOND) );
		//System.out.println("Calendar Date : " +jiso);
		//System.out.println("Calendar other Date " +ymd[0]+" "+ ymd[3]+"day   "+ date.get(Calendar.HOUR_OF_DAY) + "h " + date.get(Calendar.MINUTE) + "min  " + date.get(Calendar.SECOND)+"sec" );

		return jiso;
	}

	/**
	 * this methods look if the column of proper motion (ra or dec) exist. if it exist testPm return this value else return 0
	 * @param t1 index column which contain the RA propermotion
	 * @param t2 index column which contain the DEC propermotion
	 * @param o Aladin object like row of catalog
	 * @return double[] a table of double containing  the value of proper motion or 0
	 */
	public double[] testPm(int t1, int t2,Obj o){
		double remplacepm[]={0,0}  ;
		
		//System.out.println(t1+" "+t2+" "+v1+" "+v2);
		verifIsNumber(o, t1, "0.0");
		verifIsNumber(o, t2, "0.0");
		String v[] =o.getValues();
		if (t1!=-1){remplacepm[0] = Double.parseDouble(v[t1]);}
		if (t2!=-1){remplacepm[1] = Double.parseDouble(v[t2]);}
		
		//System.out.println(remplacepm[0]+"       "+remplacepm[1]);
		return remplacepm;
	}

	/**
	 * 
	 * @param o Aladin obj which we have to treat
	 * @param n index number of the column which the methods have to analyse
	 * @param string containing the value that methods must remove in the case in the column not contain a number
	 */
	public void verifIsNumber(Obj o, int n, String string){
		boolean valid;

		String val[] = o.getValues();

		try{valid=true;
			double nb = Double.parseDouble(val[n]);
		} catch (NumberFormatException e) {
			valid=false;
		}
		if(!valid){
			o.setValue(n, string);
		}

		Aladin.trace(3,"valeurs : -"+val[n]+"-");
	}





	/**
	 * 
	 * @param pm index number of the column containg the value of the pm vector
	 * @param posAng index number of the angle of pm vector
	 * @return int[] the new indexs of new columns containing the calculated value of PmRa and PmDec
	 * @throws AladinException
	 */
	public int[] createColumnVectPmPosAngtoRaDec(int pm, int posAng) throws AladinException{

		AladinData ad = getCat(getNome());
		System.out.println("catalogue : "+getNome());
		//ici pm et posAng ne sont pas des valeurs mais le numero de la colonne correspondante

		int n = ad.getNbObj();
		//if( n==0 ) return null;
		System.out.println("nombre d'obj : "+n);
		// Access to the catalog objects
		Obj[] ob = ad.seeObj();

		// Add a new column on one object of this catalog
		// => these metadata will be automatically set for all other objects of the same table
		Obj source1 = ob[0];
		int pos = source1.getSize();
		source1.setColumn(pos,"_PMRA","mas/yr","pos.pm;pos.eq.ra",15);
		source1.setColumn((pos+1),"_PMDEC","mas/yr","pos.pm;pos.eq.dec",15);

	
		for( int i=0; i<n; i++ ) {

			String valobj[] = ob[i].getValues();
			boolean valid;
			double myPm = 0, myPmPA = 0;
			try {
				valid=true;
				myPm = Double.parseDouble(valobj[pm]);
				myPmPA = Double.parseDouble(valobj[posAng]);
				Aladin.trace(2, "pm originaux: "+valobj[pm]+"-"+valobj[posAng]+"-");
			} catch (NumberFormatException e) {
				// can not parse pm pmPA values
				valid=false;
				ob[i].setValue(pos, "0.");
				ob[i].setValue(pos+1, "0.");
			}
			if (valid){
				//TODO parsing propre des unites 
				ob[i].setValue(pos,  Double.toString(myPm*Math.sin(Math.toRadians(myPmPA))/1000.) );
				ob[i].setValue(pos+1,Double.toString(myPm*Math.cos(Math.toRadians(myPmPA))/1000.) );
				Aladin.trace(2, "computed pmradec: "+ ob[i].getValues()[pos]);
				//ob[i].setValue((pos+1),pm*cos(posAng))
				//ob[i].setSelected(true) ;
			} 
		}

		// Notify to Aladin that catalog plane has been modified
		ad.objModified();
		int nbc[]={pos,pos+1};
		return (nbc);

	}
	/**
	 * 
	 * @param o Aladin Object, star
	 * @param n index number of column which can use to calculated the new column
	 * @param factprop a double containing just the number which multiply the other number
	 * @param endformat a string containing the unit of the new column
	 * @return index number of new columns containing the calculated value
	 * @throws AladinException
	 */
	public int mimetiColumn(Obj o, int n, double factprop,String endformat) throws AladinException{
		AladinData ad = getCat(getNome());                                           //cherche le catalogue selectionné
		int nb = ad.getNbObj();                                                      // nb est le nombre d'obj contenue dans la catalog
		Obj[] ob = ad.seeObj();                                                      //ob correspond a un tableau d'objet que sont chaque "etoiles"
		Obj source1 = ob[0];                                                         //source1 est le premier objet de l'objet ob
		int pos = source1.getSize();                                                  //pos est le nombre de colonne de l'objet source1, il va servir d'indice pour la nouvelle colonne car les indices commence à 0
		source1.setColumn(pos,"calculated by col"+n+"column en "+endformat,endformat,"UCD",15);   //edit les metadatas de la nouvelle colonne
		//voir a intergrer  verifColVal()
		for( int i=0; i<nb; i++ ) {                                                   //on itere pour integrer de nouvelles valeurs à tous les objets de notre objet ob dans la nouvelle colonne
			String valobj[] = ob[i].getValues();
			ob[i].setValue(pos,Double.toString(Double.parseDouble(valobj[n])*factprop) );
			//ob[i].setValue((pos+1),pm*cos(posAng))
			ob[i].setSelected(true) ;
		}
		n=pos;
		return n;
	}
	/**
	 * 
	 * @param o
	 * @param pmra
	 * @param pmdec
	 * @param endformat
	 * @return
	 * @throws AladinException
	 * @throws ParseException
	 */
	public int[] ctrlunits(Obj o, int pmra, int pmdec, String endformat) throws AladinException, ParseException{
		DateTimeZone.setDefault(DateTimeZone.UTC);
		String units[] = o.getUnits();
		String val[] = o.getValues();
		int npmra = pmra;
		int npmdec = pmdec;

		Unit uPMRA = new Unit(val[pmra]+units[pmra]);
		Unit uPMRAref = new Unit("1mas/yr");

		Aladin.trace(2,"units -"+uPMRA.symbol+"-");
		Aladin.trace(2,"units of ref -"+uPMRAref.symbol+"-");



		if(!uPMRA.symbol.equals(uPMRAref.symbol)){
			System.out.println("dans le if");
			uPMRAref.convertFrom(uPMRA);
			AladinData ad = getCat(getNome());                                           //cherche le catalogue selectionné
			int nb = ad.getNbObj();                                                      // nb est le nombre d'obj contenue dans la catalog
			Obj[] ob = ad.seeObj();                                                      //ob correspond a un tableau d'objet que sont chaque "etoiles"
			Obj source1 = ob[0];                                                         //source1 est le premier objet de l'objet ob
			int pos = source1.getSize();                                                  //pos est le nombre de colonne de l'objet source1, il va servir d'indice pour la nouvelle colonne car les indices commence à 0
			source1.setColumn(pos,"col"+pmra+1+" in "+units[pmra]+" into "+uPMRAref.symbol,uPMRAref.symbol,"NO",30);//edit les metadatas de la nouvelle colonne
			//voir a intergrer  verifColVal()
			for( int i=0; i<nb; i++ ) {   //on itere pour integrer de nouvelles valeurs à tous les objets de notre objet ob dans la nouvelle colonne
				String valo[] = ob[i].getValues();
				uPMRA = new Unit(valo[pmra]+units[pmra]);
				uPMRAref.convertFrom(uPMRA);
				ob[i].setValue(pos,Double.toString(uPMRAref.value) );

			}
			npmra=pos;
		}

		Aladin.trace(1,"first pmra in mas/yr: "+uPMRAref.value+" in "+uPMRAref.symbol+" from "+units[pmra]);
		//System.out.println("pmra : "+val[pmra]);


		Unit uPMDe = new Unit(val[pmdec]+units[pmdec]);
		Unit uPMDeref = new Unit("1mas/yr");

		Aladin.trace(2,"units -"+uPMDe.symbol+"-");
		Aladin.trace(2,"units of ref -"+uPMDeref.symbol+"-");

		if(!uPMDe.symbol.equals(uPMDeref.symbol)){
			uPMDeref.convertFrom(uPMDe);
			AladinData ad = getCat(getNome());                                           //cherche le catalogue selectionné
			int nb = ad.getNbObj();                                                      // nb est le nombre d'obj contenue dans la catalog
			Obj[] ob = ad.seeObj();                                                      //ob correspond a un tableau d'objet que sont chaque "etoiles"
			Obj source1 = ob[0];                                                         //source1 est le premier objet de l'objet ob
			int pos = source1.getSize();                                                  //pos est le nombre de colonne de l'objet source1, il va servir d'indice pour la nouvelle colonne car les indices commence à 0
			source1.setColumn(pos,"col"+pmdec+1+" in "+units[pmdec]+" into "+uPMDeref.symbol,uPMDeref.symbol,"NO",30);//edit les metadatas de la nouvelle colonne
			//voir a intergrer  verifColVal()
			for( int i=0; i<nb; i++ ) {                                                   //on itere pour integrer de nouvelles valeurs à tous les objets de notre objet ob dans la nouvelle colonne
				String valo[] = ob[i].getValues();
				uPMDe = new Unit(valo[pmdec]+units[pmdec]);
				uPMDeref.convertFrom(uPMDe);
				ob[i].setValue(pos,Double.toString(uPMDeref.value) );

			}
			npmdec=pos;
		}
		Aladin.trace(1,"first pmde in mas/yr: "+uPMDeref.value+" in "+uPMDeref.symbol+" from "+units[pmdec]);
		//		System.out.println("pmdec : "+val[pmdec]);
		int pm[] = {npmra,npmdec};


		return pm ;
	}

	/** 
	 * @param o	Un objet d'un plan catalogue Aladin
	 * @return	Tableau tab avec incices des colonnes
	 * 			tab[0] : ra ||
	 * 			tab[1] : dec ||
	 *          tab[2] : pmra ||
	 *          tab[3] : pmdec
	 * @throws AladinException 
	 * */
	public int[] getNumbercol(Obj o) throws AladinException{
		int iRa    = -1;  // indice de la colonne de la position J2000 RA
		int iDec   = -1;  // idem pour DEC
		int iPmRa  = -1;  // idem pour pm RA
		int iPmDec = -1;

		// Est-ce qu'on a nos colonnes calculees pour PM ?
		String name[] = o.getNames();
		for( int i=0; i<name.length; i++ ){
			if( name[i].equals("_PMRA") )     iPmRa=i;
			if( name[i].equals("_PMDEC") )    iPmDec=i;
		}
		if (iPmRa!=-1 && iPmDec!=-1) {
			Aladin.trace(2, "Found columns _PMRA and _PMDEC");
		}

		// On essaye par UCD
		String ucd[] = o.getUCDs();
		for( int j=0; j<ucd.length; j++ ) {
			if( ucd[j].equals("pos.eq.ra;meta.main")  && iRa==-1  ) iRa=j;
			if( ucd[j].equals("pos.eq.dec;meta.main") && iDec==-1 ) iDec=j;
			if( ucd[j].equals("pos.pm;pos.eq.ra") && iPmRa==-1 )     iPmRa=j;
			if( ucd[j].equals("pos.pm;pos.eq.dec") && iPmDec==-1 )    iPmDec=j;
		}
		Aladin.trace(3,"afetr UCD first Colonnes : RA="+iRa+" DEC="+iDec+" PMRA="+iPmRa+" PMDEC="+iPmDec);	
		if (iRa==-1) {
			for( int j=0; j<ucd.length; j++ ) {
				if( ucd[j].equals("pos.eq.ra") ) iRa=j;
			}
		}

		if (iDec==-1) {
			for( int j=0; j<ucd.length; j++ ) {
				if( ucd[j].equals("pos.eq.dec") ) iDec=j;
			}
		}

		Aladin.trace(2,"pos pm Colonnes : RA="+iRa+" DEC="+iDec+" PMRA="+iPmRa+" PMDEC="+iPmDec);	
		if (iPmRa==-1 && iPmDec==-1) {
			Aladin.trace(2,"pos pm Colonnes : RA="+iRa+" DEC="+iDec+" PMRA="+iPmRa+" PMDEC="+iPmDec);	
			for( int r=0; r<ucd.length; r++ ) {
				if( ucd[r].equals("pos.pm") && iPmRa==-1 ){
					iPmRa=r;
				}
				if ( ( ucd[r].equals("pos.pm;pos.posAng") || ucd[r].equals("pos.posAng;pos.pm") ) && iPmDec==-1){
					iPmDec=r;
				}
			}
			for( int i=0; i<name.length; i++  ) {
				if( name[i].equals("pm") && iPmRa==-1 ){
					iPmRa=i;
				}
				if ( name[i].equals("pmPA") && iPmDec==-1){
					iPmDec=i;
				}
			}
			Aladin.trace(2,"pos pm Colonnes : RA="+iRa+" DEC="+iDec+" PMRA="+iPmRa+" PMDEC="+iPmDec);	
			if (iPmRa!=-1 && iPmDec!=-1){
				int npm[] =createColumnVectPmPosAngtoRaDec(iPmRa, iPmDec);
				iPmRa=npm[0];
				iPmDec=npm[1];
			}	
		}


		// On essaye par noms de colonnes
		if (iRa==-1) {
			for( int i=0; i<name.length; i++ ) {
				if( name[i].equals("_RAJ2000") && iRa==-1 ) iRa=i;
			}
		}
		if (iDec==-1 ) {
			for( int i=0; i<name.length; i++ ) {
				if( name[i].equals("_DEJ2000") && iDec==-1) iDec=i;
			}
		}
		if (iPmRa==-1||iPmDec==-1){
			for( int i=0; i<name.length; i++ ){
				if( name[i].equals("pmRA") )     iPmRa=i;
				if( name[i].equals("pmDE") )    iPmDec=i;
			}
		}

		Aladin.trace(1,"In getNumberCol: Colonnes : RA="+iRa+" DEC="+iDec+" PMRA="+iPmRa+" PMDEC="+iPmDec);	


		int tab[] = {iRa,iDec,iPmRa,iPmDec};
		return tab ;
	}

	public void newCoords() throws ParseException {
		System.out.println("je suis dans newCoords");
		boolean first=true;
		/*
		int iRa    = -1;  // indice de la colonne de la position J2000 RA
		int iDec   = -1;  // idem pour DEC
		int iPmRa  = -1;  // idem pour pm RA
		int iPmDec = -1;  // idem pour pm DEC
		 //*/

		int tab[] = {-1,-1,-1,-1};

		Astroframe srcFrame = new FK5();  // Pour le moment on consid�re que les coordonn�es sont fournies en J2000
		Astroframe trgFrame = new FK5();
		//trgFrame.setFrameEpoch( getEpoch() ); // On change l'�poque d'arriv�e
		//RecupData data = new RecupData();

		try {

			//recupère le plan selectionné dans le menu deroulant
			AladinData ad = getCat(getNome());

			//data.recupData(ad);
			double outdate=RecupYear.recupYear();
			AstroEpoch astroEpoch = new AstroEpoch();

			System.out.println("avant while, outdate=" + outdate);

			// Itération sur toutes les sources du plan
			Iterator<Obj> it = ad.iteratorObj();

			while( it.hasNext() ) {
				Obj o = it.next();

				// recherche de la position des colonnes dont les noms sont PMRA et PMDEC
				// sur le premier objet récupéré (tous les autres seront supposés avoir les mêmes noms de colonnes)

				//	String units[] = o.getUnits();
				//   String name[] = o.getNames();
				if( first ) {
					first = false;

					// find column indices
					tab = getNumbercol(o);

					// If RA or DEC not found, exit
					if (tab[0]==-1 || tab[1]==-1) {
						Aladin.trace(0, "!!! No RA DEC found - can not recompute coo");
						return;
					}

					if(tab[2]!=-1 && tab[3]!=-1){
							int ntb[]=ctrlunits(o,tab[2],tab[3],"1mas/yr");
							tab[2]=ntb[0];
							tab[3]=ntb[1];
					}
					System.out.println(tab[0]+" "+tab[1]+" "+tab[2]+" "+tab[3]);		
				}

				// Récupération des coordonnées et des mouvements propres

				String val[] = o.getValues();
				double rpm[] =testPm(tab[2],tab[3],o);

				String[] newCoords = {"0","0"};

				boolean valid;
				double myRa = 0, myDec = 0;
				try {
					valid=true;
					myRa = Double.parseDouble(val[tab[0]]);
					myDec = Double.parseDouble(val[tab[1]]);
				} catch (NumberFormatException e) {
					// can not parse pm pmPA values
					valid=false;
					continue;
				}
				Aladin.trace(3, "ra -"+val[tab[0]]+"-  dec : -"+val[tab[1]]+"- pmra : -"+rpm[0]+"-  pmdec : -"+rpm[1]+"-");
				newCoords = astroEpoch.newCoords(myRa,myDec, rpm[0], rpm[1], 2000.0, outdate);
				
/*				if(tab[2]==-1 && tab[3]==-1){
					newCoords = astroEpoch.newCoords(Double.parseDouble(val[tab[0]]),Double.parseDouble(val[tab[1]]), rpm[0], rpm[1], 2000.0, outdate);
					}
				else {
					verifIsNumber(o, tab[0], "0.0");
					verifIsNumber(o, tab[1], "0.0");
					//if (!val[tab[2]].trim().contentEquals("")){
						//if(!val[tab[3]].trim().contentEquals("")){
					Aladin.trace(2, "ra -"+val[tab[0]]+"-  dec : -"+val[tab[1]]+"- pmra : -"+rpm[0]+"-  pmdec : -"+rpm[1]+"-");
							newCoords = astroEpoch.newCoords(Double.parseDouble(val[tab[0]]),Double.parseDouble(val[tab[1]]), rpm[0], rpm[1], 2000.0, outdate);
						//}
					//}
				}
*/
				//else {newCoords = astroEpoch.newCoords(Double.parseDouble(val[tab[0]]),Double.parseDouble(val[tab[1]]), 0, 0, 2000.0, outdate);}

				/*
				System.out.println("");
				System.out.println("les unités");
				for( int j=0; j<units.length; j++ ) {
					System.out.println("col:"+j+"name:"+name[j]+"units :"+units[j]);
				}
				//*/



				double ra = Double.parseDouble(newCoords[0]);
				double dec = Double.parseDouble(newCoords[1]);

				// Recalcul de la position en fonction de la nouvelle époque
				Astrocoo coo = new Astrocoo(srcFrame,ra,dec);
				coo.convertTo(trgFrame);
				ra=coo.getLon();
				dec=coo.getLat();

				// Affectation de la nouvelle position à l'objet
				o.raj = ra;
				o.dej = dec;

			}

			// On dit à Aladin qu'on a modifié les objets et qu'il faut qu'il réaffiche
			ad.objModified();


		}
		catch( AladinException e ) { 
			aladin.warning("Plugin error: "+e.getMessage()+"\n" +
			"You have to selected the concerned catalog plane !");
		}

	}	
}

